from .parser import DefaultParser


class Database(object):
    def __init__(self, species: list, elements: dict, parser=None):
        self.elements = elements
        self.species = species
        self.parser = parser

    def __str__(self):
        padding = 8
        print(f"{'Database':{'-'}{'^'}{64}}")
        print(
            f"{'Species':{padding}}\t{'dGf':>{padding}}\t{'dHf':>{padding}}\t{'Sm':>{padding}}"
        )
        print(f"{'':{'-'}{'^'}{64}}")
        for species in self.species:
            print(species)
        return f"{'':{'-'}{'^'}{64}}"

    @classmethod
    def from_default(cls, species: tuple):
        """Create database containing species and their chemical properties from a tuple of string
        : param species : Species data as tuple of strings
        : type species : tuple
        """
        elements = DefaultParser.parse_elements()
        species = DefaultParser.parse_species(species, elements)
        return cls(species, elements, DefaultParser)

    @classmethod
    def from_file(cls, filename: str):
        """Create database containing species and their chemical properties from a txt file
        : param filename : file containing species data
        : type filename : str
        """

        with open(filename, "r", encoding="utf-8") as f:
            lines = f.read().splitlines()
        return cls.from_default(tuple(lines))

    def add_reactions(self, reactions: tuple):
        parsed_reactions = self.parser.parse_reactions(reactions)
        return self.associate_reactions_to_database(parsed_reactions)

    def associate_reactions_to_database(self, reactions: list):
        complete = []
        for reac in reactions:
            dic = dict()
            for key, value in reac.items():
                for react in self.species:
                    if key == react.formula:
                        dic[react] = value
            complete.append(dic)
        return complete


elements = {
    "e": 0,
    "La": 0,
    "H": 1.0079,
    "Xe": 131.3,
    "He": 4.0026,
    "Li": 6.941,
    "Rf": 265.0,
    "B": 10.81,
    "Db": 268.0,
    "Fe": 55.847,
    "Sg": 271.0,
    "Bh": 272.0,
    "F": 18.998,
    "Ag": 107.87,
    "Mt": 276.0,
    "Na": 22.99,
    "Mg": 24.305,
    "Cs": 132.91,
    "Al": 26.982,
    "Zr": 91.224,
    "Si": 28.086,
    "P": 30.974,
    "S": 32.06,
    "Cl": 35.453,
    "Ar": 39.948,
    "Ca": 40.08,
    "N": 14.007,
    "Sc": 44.955912,
    "Hf": 178.49,
    "Ti": 47.9,
    "Hg": 200.59,
    "V": 50.941,
    "Cr": 51.996,
    "Ta": 180.95,
    "O": 15.999,
    "Mn": 54.938,
    "Be": 9.0122,
    "Ba": 137.33,
    "Co": 58.933,
    "Fr": 223.0,
    "Os": 190.2,
    "Cu": 63.546,
    "Ir": 192.22,
    "Zn": 65.38,
    "Pt": 195.09,
    "Au": 196.97,
    "Ga": 69.72,
    "Ge": 72.59,
    "C": 12.011,
    "As": 74.922,
    "W": 183.84,
    "Tl": 204.37,
    "Bi": 206.98,
    "Br": 79.904,
    "Rn": 222.0,
    "At": 210.0,
    "Kr": 83.8,
    "Rb": 85.468,
    "Sr": 87.62,
    "Y": 88.906,
    "Pb": 207.2,
    "Nb": 92.906,
    "Ni": 58.71,
    "Ne": 20.179,
    "Mo": 95.94,
    "Tc": 97.0,
    "Ra": 226.03,
    "Ac": 227.0,
    "Ru": 102.91,
    "Po": 209.0,
    "Re": 128.207,
    "Pd": 106.4,
    "K": 39.096,
    "Cd": 112.41,
    "In": 114.82,
    "Sn": 118.69,
    "Sb": 121.75,
    "Se": 78.96,
    "I": 126.9,
    "Te": 127.6,
}

element_list = list(elements.keys())
